// 定时任务 ID（用于清除任务）
var crontaskTime = 3000
let cronTaskId = null;

// 初始化任务配置
function initializeCronSettings() {
    // 从 LocalStorage 获取任务状态，默认启用
    const cronEnabled = JSON.parse(localStorage.getItem('cronEnabled')) ?? true;
    document.getElementById('enableCronCheckbox').checked = cronEnabled;

    // 根据状态启用或禁用任务
    if (cronEnabled) {
        startCronTask();
    }
}


function startCronTask() {
    if (cronTaskId !== null) return; // 防止重复启动

    cronTaskId = setInterval(() => {
        const pendingTasks = JSON.parse(localStorage.getItem('pendingTasks')) || [];


        // ✅ 控制最大并发任务数
        if (pendingTasks.length > 0 && runningTasks < maxConcurrentTasks) {
            const task = pendingTasks.shift();
            runningTasks++; // ✅ 增加当前并发数

            // 开始异步任务
            jobdetail(task).then(() => {
                runningTasks--;
            }).catch(err => {
                console.error('任务失败：', err);
                runningTasks--; // 防止阻塞
            });

            // 更新任务队列
            localStorage.setItem('pendingTasks', JSON.stringify(pendingTasks));
        } else {
            if (runningTasks >= maxConcurrentTasks) {
                console.log(`达到最大并发数 (${runningTasks}/${maxConcurrentTasks})，等待空闲...`);
            } else {
                console.log('No tasks to process.');
                if (runningTasks == 0) {
                    mlog(runningTasks,'runningTasks')
                    // clearInterval(cronTaskId);
                }
            }
        }

    }, crontaskTime);
}


async function jobdetail(task) {
    // job
    console.log(`开始任务：${task.tasktime} - ${JSON.stringify(task.errorprocess)}`);
    // await new Promise(resolve => setTimeout(resolve, 10000)); // 模拟耗时任务

    if (task['save_var'] == 'localstorage') {
        let cachep = JSON.stringify(defaultPostsSetting)
        let postsetting = localStorage.getItem('postsSave') ?? cachep
        postsetting = JSON.parse(postsetting)
        save = postsetting
    }
    // else if (task['save_var'] == '001') {
    //     save='00'
    // }
    else {
        save = task['save_var']
    }

    let prompt = task['job']
    let airespon = null
    try {
        var bywho = contentAISEOAdmin.usr
        
        let result = await mergePromptSend(prompt, save, returncounts = 1, jsonformat = '11')
        airespon = result['respond']
        // throw new Error('Something went wrong');
        //记录到任务列表当作
        const db = new PromptDatabase();
        let postid = airespon['post_id']
        let title = airespon['article_title']

        const jobId = await db.addJobLog({
            product_by_who: bywho,
            post_id: postid,
            post_title: title,
            post_time: new Date().toISOString()
        });

        if (task['errorprocess'] != '00') { 
            if (task['errorprocess']['type'] == 'rg') {
                let tableData = outtable.cache['ID-table-prompt-setting'];
                modifybulkTableCell(tdata = tableData, itemid = task['errorprocess']['id'], finishstatus = 'complete', tableid = 'ID-table-prompt-setting')
            }
        }

        console.log(`完成任务：${task.tasktime}`);
        console.log(`任务内容：${JSON.stringify(airespon)}`);

    } catch (error) {
        //默认处理错误
        if (task['errorprocess'] == '00') {
            const jobId = await db.addJobLog({
                product_by_who: bywho,
                post_id: 'postid',
                post_title: airespon,
                post_time: new Date().toISOString()
            });
        }
        else {
            if (task['errorprocess']['type'] == 'rg') {
                let tableData = outtable.cache['ID-table-prompt-setting'];
                modifybulkTableCell(tdata = tableData, itemid = task['errorprocess']['id'], finishstatus='failed', tableid = 'ID-table-prompt-setting')
            }

            console.log(`任务内容：${JSON.stringify(airespon)}`);

        }
    }
}

// 停止定时任务
function stopCronTask() {
    if (cronTaskId !== null) {
        clearInterval(cronTaskId);
        cronTaskId = null;
    }
}

// 更新任务配置状态
function updateCronSetting(enabled) {
    localStorage.setItem('cronEnabled', JSON.stringify(enabled));
    if (enabled) {
        startCronTask();
    } else {
        stopCronTask();
    }
}

// 添加示例任务
function addContentTask(createContentPrompt_, showtip_target, save_var = 'localstorage', errorprocess = "00") {
    let createContentPrompt = createContentPrompt_
    const pendingTasks = JSON.parse(localStorage.getItem('pendingTasks')) || [];
    let now = new Date();
    // console.log(now.toLocaleString());
    mlog(errorprocess,'add task')
    const ttime = now.toLocaleString();
    pendingTasks.push({ tasktime: ttime, job: createContentPrompt, save_var: save_var, errorprocess: errorprocess });
    localStorage.setItem('pendingTasks', JSON.stringify(pendingTasks));
    // alert(`Task taskId added!`);
    atip(`Task added`, showtip_target)
}

// 查看任务状态
function viewTasks() {
    var status = 'Off'
    if (contentTaskRunning > 0) {
        status = 'Running'
    }
    const statusTask = { 'Cron Task of runningtasks counts': runningTasks, 'StatusTask': status }
    const output = `
    <h3>Tasks:</h3><p>${JSON.stringify(statusTask, null, 2)}</p>
    `;
    document.getElementById('taskOutput').innerHTML = output;
}

function modifybulkTableCell(tdata, itemid, finishstatus, tableid = 'ID-table-prompt-setting') {
    var newData = tdata.map(item => {
        if (item.id === itemid) {
            item.status = finishstatus; // 修改单元格内容
        }
        return item;
    });

    // 刷新表格
    outtable.reload(tableid, {
        data: newData
    });
}

// 初始化页面
initializeCronSettings();

let runningTasks = 0; // 当前并发任务数
const maxConcurrentTasks = 1;//最大任务并发数量